%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: debug_unify.pl,v $
%%  $Revision: 1.2 $
%%      $Date: 2003/05/19 16:35:58 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: Code for computing a unification failure of two feature
%%             structures.
% GBP - what on earth does this mean?
%%             This code is based on diff.pl by Detmar Meurers
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: debug_unify.pl,v $
%% Revision 1.2  2003/05/19 16:35:58  stefan
%% added noadderrs around add_to/3 call in debug_unify_input/6
%%
%% Revision 1.1  2003/05/10 13:48:52  stefan
%%
%% Revision 1.1  2003/05/10 12:16:49  stefan
%% Initial revision
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(lists),[select/3]). % and nonmember/2 from sics.pl
:- use_module(library(assoc),[assoc_to_list/2,put_assoc/4,del_assoc/4,
                              empty_assoc/1,get_assoc/3]).

% ------------------------------------------------------
% top-level predicate, using results saved using save_results_on
% as defined in save_results.pl

debug_unify(NrOrDescA,NrOrDescB) :-
   debug_unify(NrOrDescA,[],NrOrDescB,[]).

% Paths can be entered either as f:g:h:i or as [f,g,h,i]
% This does not take into account potential residues yet!
debug_unify(NrOrDescA,PathA,NrOrDescB,PathB) :-
   get_debug_unify_input(NrOrDescA,PathA,FSA,_ResidueA,DupsA,PathsA),
   get_debug_unify_input(NrOrDescB,PathB,FSB,_ResidueB,DupsB,PathsB),
   debug_unify(FSA,FSB,IdentA,IdentB,UnfA,UnfB,DupsA,DupsB,PathsA,PathsB),
   pref_struc_on,   % we want to see debug_unifys as structures (first) not as trees
   set_title(debug_unify:NrOrDescA),
   quiet_save_results_off,	% don't save again what we've already got
   grisu_pp_fs(FSA,flagAss(IdentA,UnfA)),
   set_title(debug_unify:NrOrDescB),
   grisu_pp_fs(FSB,flagAss(IdentB,UnfB)),
   save_results_restore,	% switch back to previous value
   clear_title,
   pref_struc_restore.

% --------------------------------
% auxiliary:

get_debug_unify_input(NrOrDesc,Path,FS,Residue,Dups,Paths) :-
   (   number(NrOrDesc)
   ->  saved_result(NrOrDesc,Desc)
   ;   Desc=NrOrDesc),
   call_residue((secret_noadderrs_toggle(Mode),
                 add_to(Desc,NewFS),
		 % deref(Tag,bot,Ref0,SVs0),
		 % extensionalise(Ref0,SVs0),
                 secret_adderrs_toggle(Mode)),
		Residue),
   path_as_list(Path,PathList),
   deref(NewFS,TFS,Type,AttPos),
   pathval(PathList,NewFS,AttPos,Type,TFS,FS,bot),
   sharing_paths_assocs(FS,Dups,Paths).

% --------------------------------
% debug_unify/10: the main predicate doing the work
%

debug_unify(FSA,FSB,IdentA,IdentB,UnfA,UnfB,DupsA,DupsB,PathsA,PathsB) :-
   empty_assoc(Empty),
   debug_unify_fs(FSA,FSB,Empty,_Vis,
           Empty,IdentA,Empty,IdentB,
           Empty,UnfA,Empty,UnfB,
           [],_,
           DupsA,DupsB,PathsA,PathsB).

debug_unify_fs(FSA,FSB,Vis0,Vis,
	IdentA0,IdentA,IdentB0,IdentB,
        UnfA0,UnfA,UnfB0,UnfB,
	Debug_unify0,Debug_unify,
        DupsA,DupsB,PathsA,PathsB) :-
   % deref(FSA,RefA,SVsA),
   % deref(FSB,RefB,SVsB),
   (   get_assoc(FSA,Vis0,Debug_unify0-Debug_unify1) % Seen this node before?
   ->  % must be a tag:
       Vis=Vis0,IdentA1=IdentA0,IdentB1=IdentB0,
       UnfA1=UnfA0,UnfB1=UnfB0,Debug_unify=Debug_unify1
   ;   % unseen node (struc or tag):
       put_assoc(FSA,Vis0,X-X,Vis1),      % remember seeing this node
       fs2keyed_feats(FSA,KFeatsA,TypeA), % get features+type of node
       fs2keyed_feats(FSB,KFeatsB,TypeB), %         ""
       debug_unify_type(TypeA,TypeB,Debug_unify0,Debug_unify1),
%        % check sharings if node is a tag in at least one of the structures:
%        (   get_assoc(RefA,DupsA,SharingPathsA) 
%        ->  debug_unify_reentrancy(SharingPathsA,DupsB,PathsB,Debug_unify1,Debug_unify2)
%        ;   (   get_assoc(RefB,DupsB,SharingPathsB)
% 	   ->  debug_unify_reentrancy(SharingPathsB,DupsA,PathsA,Debug_unify1,Debug_unify2)
% 	   ;   Debug_unify2=Debug_unify1
% 	   )
%        ),
       Debug_unify2=Debug_unify1,
       debug_unify_fs_feats(KFeatsA,KFeatsB,Vis1,Vis2,
                     IdentA0,IdentA1,IdentB0,IdentB1,
                     UnfA0,UnfA1,UnfB0,UnfB1,
                     Debug_unify2,Debug_unify,
                     DupsA,DupsB,PathsA,PathsB),
       remember_tag_debug_unify(FSA,DupsA,Debug_unify0,Debug_unify,Vis2,Vis)
   ),
   % If something is locally inconsistent (i.e. Debug_unify0 =/= Debug_unify1?)
   % we need to show this node in red. IdentA/B contain all other nodes.
   mark_identical(Debug_unify0,Debug_unify2,FSA,IdentA1,IdentA,
		              FSB,IdentB1,IdentB),
   % If something that's part of this structure or its substructure
   % changed (i.e., Debug_unify0 =/= Debug_unify) then we need to show this substructure
   mark_identical(Debug_unify0,Debug_unify,FSA,UnfA1,UnfA,
                             FSB,UnfB1,UnfB).

% Remember if this is a tag and inside not everything identical:
remember_tag_debug_unify(FSA,DupsA,Debug_unify0,Debug_unify,Vis0,Vis) :-
   get_assoc(FSA,DupsA,_), !,
   (   Debug_unify0==Debug_unify
   ->  Vis=Vis0
   ;   del_assoc(FSA,Vis0,_,Vis1),
       put_assoc(FSA,Vis1,X-[debug_unify_below_tag|X],Vis)
   ).
remember_tag_debug_unify(_,_,_,_,Vis,Vis).

/*
We do not care about reantrancies in unification
debug_unify_reentrancy(SharingPathsX,DupsY,PathsY,Debug_unify0,Debug_unify) :-
  % SharingPathsX = list of paths that this node structure shares with
  % Pick one, to get its tag in structure Y
   SharingPathsX=[SharingPath|_], 
   (  (get_assoc(SharingPath,PathsY,SharingRefY), 
       get_assoc(SharingRefY,DupsY,SharingPathsY))
   -> true
   ;  SharingPathsY=[] ),
   % Iff the same directories share in X and Y, it's no debug_unifyerence
   (   SharingPathsX=SharingPathsY 
   ->  Debug_unify=Debug_unify0
   ;   %format(user_output,'~p ~p~n',[SharingPathsX,SharingPathsY]),
       Debug_unify=[structure_sharing|Debug_unify0]
   ).
*/


debug_unify_type(TypeA,TypeB,Debug_unify0,Debug_unify) :-
   (  unify_type(TypeA,TypeB,_)                % diff -> TypeA==TypeB              
   -> Debug_unify=Debug_unify0                
   ;  Debug_unify=[types|Debug_unify0]         % A and B have incompatible types
   ).                           

/*
% I guess this has to be skipped since it is clear that
% the features are appropriate, their non-existence in one Desc
% cannot yield a unification failure.
debug_unify_fs_feats([],KeyedFeatsB,Vis,Vis,
             IdentA,IdentA,IdentB,IdentB,
             UnfA,UnfA,UnfB,UnfB,
             Debug_unify0,Debug_unify,
             _DupsA,_DupsB,_PathsA,_PathsB) :-
   (  KeyedFeatsB==[]
   -> Debug_unify=Debug_unify0
   ;  Debug_unify=[b_feats|Debug_unify0]              % B has features not in A
   ).
*/
debug_unify_fs_feats([],_KeyedFeatsB,Vis,Vis,
             IdentA,IdentA,IdentB,IdentB,
             UnfA,UnfA,UnfB,UnfB,
             Debug_unify,Debug_unify,
             _DupsA,_DupsB,_PathsA,_PathsB).


debug_unify_fs_feats([VA|VsA],FeatsB0,Vis0,Vis,
             IdentA0,IdentA,IdentB0,IdentB,
             UnfA0,UnfA,UnfB0,UnfB,
             Debug_unify0,Debug_unify,
             DupsA,DupsB,PathsA,PathsB):-
   debug_unify_feat(VA,FeatsB0,FeatsB1,Vis0,Vis1,
             IdentA0,IdentA1,IdentB0,IdentB1,
             UnfA0,UnfA1,UnfB0,UnfB1,
             Debug_unify0,Debug_unify1,
             DupsA,DupsB,PathsA,PathsB),
   debug_unify_fs_feats(VsA,FeatsB1,Vis1,Vis,
               IdentA1,IdentA,IdentB1,IdentB,
               UnfA1,UnfA,UnfB1,UnfB,
               Debug_unify1,Debug_unify,
               DupsA,DupsB,PathsA,PathsB).

debug_unify_feat(fval(F,VA,_),FeatsB0,FeatsB,Vis0,Vis,
          IdentA0,IdentA,IdentB0,IdentB,
          UnfA0,UnfA,UnfB0,UnfB,
          Debug_unify0,Debug_unify,
          DupsA,DupsB,PathsA,PathsB) :-
   select(fval(F,VB,_),FeatsB0,FeatsB), !,
   debug_unify_fs(VA,VB,Vis0,Vis,
           IdentA0,IdentA,IdentB0,IdentB,
           UnfA0,UnfA,UnfB0,UnfB,
           Debug_unify0,Debug_unify,
           DupsA,DupsB,PathsA,PathsB).

/*
debug_unify_feat(_,FeatsB,FeatsB,Vis,Vis,
          IdentA,IdentA,IdentB,IdentB,
          UnfA,UnfA,UnfB,UnfB,
          Debug_unify0,[a_feats|Debug_unify0], % A has features not in B
          _DupsA,_DupsB,_PathsA,_PathsB).
*/


%% This wrongly prints a DTRs feature in Red
debug_unify_feat(_,FeatsB,FeatsB,Vis,Vis,
          IdentA,IdentA,IdentB,IdentB,
          UnfA,UnfA,UnfB,UnfB,
          Debug_unify0,Debug_unify0, 
          _DupsA,_DupsB,_PathsA,_PathsB).


/*
This wrongly prints a DTRs feature in Red
debug_unify_feat(fval(_F,_VA,_),FeatsB0,FeatsB0,Vis0,Vis,
          IdentA0,IdentA,IdentB0,IdentB,
          UnfA0,UnfA,UnfB0,UnfB,
          Debug_unify0,Debug_unify,
          DupsA,DupsB,PathsA,PathsB) :-
   debug_unify_fs(NewFS1,NewFS2,Vis0,Vis,
           IdentA0,IdentA,IdentB0,IdentB,
           UnfA0,UnfA,UnfB0,UnfB,
           Debug_unify0,Debug_unify,
           DupsA,DupsB,PathsA,PathsB).

*/