; Purpose: Emacs lisp mode for starting trale under emacs. 
;
; Author: Detmar Meurers
;
; Version: $Id: trale-emacs-interface.el,v 1.11 2005/03/31 17:34:24 stefan Exp $
;
; Historical note: We now add our stuff on top of the ordinary
; prolog.el mode that comes with sicstus. This improves on our
; original gtrale.el mode (which directly used comint) in that it
; allows use of the source level debugger and other facilities
; provided by the prolog.el mode of sicstus. Different from gtrale.el,
; this new code should also be compatible with both xemacs and gnu
; emacs.

; Load the prolog mode
(load "prolog")
(load "trale")


; (setq prolog-system 'sicstus)



; Define the start-trale function, which is called in the shell script
; "trale" that's used for starting trale

(defun start-trale (program-name options startup-filename)
; wait for emacs to get to its feet:
  (sit-for 1)
; set the variables from the arguments and start prolog
  (setq prolog-program-name program-name)
  (setq sicstus-program-switches (split-string options))
  (run-prolog nil)
; wait until we see a prolog prompt
;  (wait-until-prompt)
; make the prolog window take up the entire frame:
  (delete-other-windows)

  (setq mode-name "trale")
  (setq major-mode 'trale-interactive-mode)
  
; load startup files
  (unless (equal startup-filename "")
    (let* ((startup-load-command (concat "compile('" 
					 startup-filename
					 "').\n")))
      (prolog-process-insert-string (get-process "prolog") startup-load-command)
      (process-send-string "prolog" startup-load-command))
    
    (setq trale-grammar-name (filename-only startup-filename))
    (setq trale-grammar-directory (dirname-only startup-filename))
    
    )
 
  ;(setq local-abbrev-table prolog-mode-abbrev-table)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-mode-map)
  (prolog-inferior-menu)

  (setq trale-standalone-flag nil)

 
  (run-hooks 'trale-mode-hook))


(defun start-standalone-trale (startup-filename)
; wait for emacs to get to its feet:
  (sit-for 1)
  (setq prolog-system 'sicstus)
  (setq prolog-program-name "sicstus")
 ; make the prolog window take up the entire frame:
  (delete-other-windows) 

  (switch-to-buffer (make-comint "prolog" startup-filename))
  
  (setq mode-name "trale")
  (setq major-mode 'trale-interactive-mode)
  ;(setq local-abbrev-table prolog-mode-abbrev-table)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-mode-map)
  (prolog-inferior-menu)
;  (set-syntax-table text-mode-syntax-table)
;  (set-syntax-table prolog-mode-syntax-table)

  ; needed for fontification
  (trale-prolog-mode-variables)
  (prolog-build-case-strings)
  (prolog-set-atom-regexps)

  (setq trale-standalone-flag t)

  (run-hooks 'trale-mode-hook))


;; taken from definition of prolog-ensure-process in prolog.el
(defun wait-until-prompt ()
  (goto-char (point-max))
  (while
      (save-excursion
	(not
	 (re-search-backward
	  (concat "\\(" prolog-prompt-regexp-i "\\)" "\\=")
	  nil t)))
    (sit-for 0.1)))

; Specifications for the source level debugger (from Gerald):

(setq prolog-use-sicstus-sd t)

(setq pltrace-port-arrow-assoc '( 
	("call" . ">>>") ("exit" . "+++") ("ndexit" . "?++")
	("redo" . "<<<") ("fail" . "---") ("exception" . "==>")
        ("null" . nil)))

(setq auto-mode-alist
      (cons '("\\.pl$" . prolog-mode) auto-mode-alist))


;; for the chart display prompt
;; ^C^N -> next prompt ^C^P -> previous prompt
;; typing enter after the last input sentence parses this sentence

(setq sicstus-prompt-regexp ">>> *\\|| [ ?][- ] *")


;;                 "\\(^! \\(.*error.*$\\)?\\)\\|\\(!!! [0-9]* expected !!!\\)"


;; Do we need Prolog debugging?
;; If this is enabled, we get inferences with emacs after loading
;; a grammar, if we want to go to interactive mode immediately.
;; St. M. 13.02.2005
; GBP - NICE ... please don't disable the debugger again.
; (setq prolog-use-sicstus-sd nil)