/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#ifndef LRS_NODE_WIDGET_H
#define LRS_NODE_WIDGET_H

#include "LRS_Util.h"
#include "LRS_TreeLayout.h"

#include <qlabel.h>
#include <qstring.h>
#include <qglobal.h>

namespace LRS {

using namespace std;

class NodeWidget : public QWidget, public TreeLayout::Node, public NonCopyable
{
public:
  class Section : public QLabel {
  public:
    Section(const char* text, bool isStat, NodeWidget* parent)
      : QLabel(text, parent), _flags(isStat ? fStatic : 0)
    {}
    
    const vector<NodeWidget*>& children() const
      { return _children; }
    bool isStatic() const
      { return (_flags & fStatic   ) != 0; }
    bool hasCollapsedChild() const
      { return (_flags & fCollapsed) != 0; }
    
    void expandChildren();
    void showChildren();
    void hideChildren();
    
    void addChild(NodeWidget* w)
      { _children.push_back(w); }
    void setHasCollapsedChild()
      { _flags |= fCollapsed; }
    void clearHasCollapsedChild()
      { _flags &= ~fCollapsed; }
      
  protected:
    void mouseReleaseEvent (QMouseEvent* mouseEvent);
    
  private:
    unsigned short _flags;
    vector<NodeWidget*> _children;

    enum { fStatic = 0x1, fCollapsed = 0x2, fAll = 0xffff };
  };

private:  
  vector<Section*> _sections;
  vector<NodeWidget*> _children;
  NodeWidget* _parentNode;
  Section* _parentSection;
  bool _collapsed;
  int _treeW, _treeH;

public:  
  NodeWidget(const string& lrsExpr, QWidget* parent);  
  NodeWidget(Section* parentSection, NodeWidget* parentNodeWidget);
  
  unsigned numSections() const { return _sections.size(); }
  const Section* section(unsigned pos) const { return _sections.at(pos); }

private:  
  void collapse();
  void expand();
  void _hide();
  void _show();
  void childrenSync();
  void updateLayout();

protected:
#if QT_VERSION >= 0x40000
  void changeEvent(QEvent*);
#else
  void fontChange(const QFont&);
#endif
  void paintEvent(QPaintEvent*);
  

public:  
  // TreeLayout::Node interface
  //
  unsigned numChildren()          { return _children.size(); }
  TreeLayout::Node* child(unsigned pos) { return _children.at(pos); }
  
  int width()      { return QWidget::width(); }
  int height()     { return QWidget::height(); }
  int treeWidth()  { return _treeW; }
  int treeHeight() { return _treeH; }
  int x()          { return QWidget::x(); }
  int y()          { return QWidget::y(); }
  
  void setPosition(int x, int y) { move(x, y); }
  void setTreeSize(int w, int h) { _treeW = w; _treeH = h; }

  friend struct NodeWidgetCreatingVisitor;
  friend struct NodeWidget::Section;
};

}

#endif // !LRS_NODE_WIDGET_H
