/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#include "LRS_ObjectModel.h"
#include "LRS_ExpressionParser.h"

#include <algorithm>
#include <functional>
#include <sstream>

namespace LRS {
  using namespace std;
  
  struct XMLStringCreatingTermVisitor : TermVisitor {
    stringstream ss;
    
    XMLStringCreatingTermVisitor(Expression* expr) {
      ss << "<LRS>";
      processListOfTerms(expr);
      ss << "</LRS>";
    }
    
    XMLStringCreatingTermVisitor(const std::string& lrsExpr) {
      auto_ptr<Expression> expr(ExpressionParser::parse(lrsExpr));
      ss << "<LRS>";
      processListOfTerms(expr.get());
      ss << "</LRS>";
    }
    void processListOfTerms(const ListOfTerms* ls) {
      for_each(ls->begin(), ls->end(),
        bind2nd(mem_fun(&Term::accept), this));
    }
    void visit(Var* t) {
      ss << "<Var name='" << t->name() << "'/>";
    }
    void visit(MetaVar* t) {
      ss << "<MetaVar name='" << t->name();
      if (t->subTerms()) {
        ss << "'><subTerms>";
        processListOfTerms(t->subTerms());
        ss << "</subTerms>";
        ss << "</MetaVar>";
      }
      else {
        ss << "'/>";
      }
    }
    void visit(Functor* t) {
      ss << "<Functor name='" << t->name() << "'>";
      ss << "<arguments>";
      processListOfTerms(t->arguments());
      ss << "</arguments>";
      if (t->subTerms()) {
        ss << "<subTerms>";
        processListOfTerms(t->subTerms());
        ss << "</subTerms>";
      }
      ss << "</Functor>";
    }
    void visit(InCont* t) {
      ss << "<InCont>";
      processListOfTerms(t->innerTerms());
      ss << "</InCont>";
    }
    void visit(ExCont* t) {
      ss << "<ExCont>";
      processListOfTerms(t->innerTerms());
      ss << "</ExCont>";
    }
    void visit(SqCont* t) {
      ss << "<SqCont>";
      processListOfTerms(t->innerTerms());
      ss << "</SqCont>";
    }
  };
  
  
  string Expression::toXMLString(const std::string& lrsExpr) {
    return XMLStringCreatingTermVisitor(lrsExpr).ss.str();
  }
  
  string Expression::toXMLString() {
    return XMLStringCreatingTermVisitor(this).ss.str();
  }
}
