/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of the <code>GTypeHierarchyData</code> class.

    @author Holger Wunsch */


#include "gtypehierarchydata.h"


/** Creates a new type hierarchy data object.
    @param name the name of the type hierarchy. The name is used in the list
      of currently available data (in the main window) and as the title of
      viewer-windows that display this type hierarchy.
    @param filename the name of the SVG file where the image data of the type hierarchy
      is stored. */
GTypeHierarchyData::GTypeHierarchyData(QString& name, QString& filename) :
  name(name), filename(filename) {
}


/** Returns the name of this type hierarchy.
    @return the name. */
QString GTypeHierarchyData::getName(void) {
  return name;
}


/** Returns the name of the file that contains the SVG data of this type
    hierarchy.
    @return the filename. */
QString GTypeHierarchyData::getFilename(void) {
  return filename;
}


/** Writes the data to a file.
    @param file the stream handle of the file. */
void GTypeHierarchyData::write(FILE* file) {
  fprintf(file, "!show_svg_file \"%s\"\n", (const char*) getFilename().local8Bit());
}
