/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file
    
    This file contains the declarations for the class <code>GTypeHierarchyData</code>.

    @author Holger Wunsch */


#ifndef __GTYPEHIERARCHYDATA_H
#define __GTYPEHIERARCHYDATA_H


#include <stdio.h>
#include <qstring.h>


/** This class represents a type hierarchy. The current implementation
    contains just a name for the type hierarchy and a file name which
    indicates a file that contains SVG (Structured Vector Graphics) data
    of the type hierarchy.

    @author Holger Wunsch */
class GTypeHierarchyData {
  /** The name of this type hierarchy, which is used in the list of
      currently available data and as the title of viewer-windows displaying
      this type hierarchy. */
  QString name;


  /** The name of the SVG file where the image data of the type hierarchy
      is stored. */
  QString filename;


 public:
  /** Creates a new type hierarchy data object.
      @param name the name of the type hierarchy. The name is used in the list
        of currently available data (in the main window) and as the title of
        viewer-windows that display this type hierarchy.
      @param filename the name of the SVG file where the image data of the type hierarchy
        is stored. */
  GTypeHierarchyData(QString& name, QString& filename);


  /** Returns the name of this type hierarchy.
      @return the name. */
  QString getName(void);


  /** Returns the name of the file that contains the SVG data of this type
      hierarchy.
      @return the filename. */
  QString getFilename(void);


  /** Writes the data to a file.
      @param file the stream handle of the file. */
  void write(FILE* file);
};


#endif
