% SICStus doesn't have to_lower or to_upper

to_lower(U,L) :-
  is_upper(U) -> L is U + 32
               ; L = U.

to_upper(L,U) :-
  is_lower(L) -> U is L - 32
               ; U = L.

/*  This file is intended as an example of how trivially easy it
    is to crunch characters in Prolog, without needing a special
    string data type.  As Xerox Quintus Prolog has strings forced
    on it by Lisp, this package will work nicely with strings if
    it has to (hence the use of library(strings)).  The first
    argument of all of these predicates must be a text object.
    That is, a string (in Xerox Quintus Prolog) or an atom.

    lower(X, Y) converts X to a constant of the same type with
	all upper case letters forced to lower case, binds Y to it.
    upper(X, Y) converts X to a constant of the same type with
	all lower case letters forced to upper case, binds Y to it.

    lower(X) is true when the name of X has no upper case letters.
    upper(X) is true when the name of X has no lower case letters.

    As an extra-special fun-type not-to-be-imitated hack,
    {lower,upper}/2 will convert a **NON-EMPTY** list
    of character codes to the appropriate case pattern

    Note the emphasis on **NON-EMPTY**:  an empty list is an atom.
    In most versions of Quintus Prolog, that atom is spelled "[]".
    But in Xerox Quintus Prolog, for compatibility with Lisp, the
    empty list is spelled "NIL" which is three upper-case letters.

    The behaviour of these operations on non-ground arguments or
    arguments which are lists but whose elements are not all
    character codes is NOT DEFINED.  In this release, none of
    these predicates issues error messages in its own name.
    This package is NOT SUPPORTED.
*/


%   lower(+Text, ?Lower)
%   converts an atom, [XQP] string, or non-empty list of character codes
%   Text to lower case.  Lower and Text are the same type of term.

lower(Text, Lower) :-
	(   atom(Text) ->
		atom_chars(Text, TextChars),
		lower_chars(TextChars, LowerChars),
		atom_chars(Lower, LowerChars)
	;   /* maybe a list of character codes */
		lower_chars(Text, Lower)
	).

lower_chars(-, _) :- !, fail.
lower_chars([], []).
lower_chars([T|Ts], [U|Us]) :-
	to_lower(T, U),
	lower_chars(Ts, Us).


%   upper(+Text, ?Upper)
%   converts an atom, [XQP] string, or non-empty list of character codes
%   Text to upper case.  Upper and Text are the same type of term.

upper(Text, Upper) :-
	(   atom(Text) ->
		atom_chars(Text, TextChars),
		upper_chars(TextChars, UpperChars),
		atom_chars(Upper, UpperChars)
	;   /* maybe a list of character codes */
		upper_chars(Text, Upper)
	).

upper_chars(-, _) :- !, fail.
upper_chars([], []).
upper_chars([T|Ts], [U|Us]) :-
	to_upper(T, U),
	upper_chars(Ts, Us).

is_upper(65). is_upper(66). is_upper(67). is_upper(68). is_upper(69).
is_upper(70). is_upper(71). is_upper(72). is_upper(73). is_upper(74).
is_upper(75). is_upper(76). is_upper(77). is_upper(78). is_upper(79).
is_upper(80). is_upper(81). is_upper(82). is_upper(83). is_upper(84).
is_upper(85). is_upper(86). is_upper(87). is_upper(88). is_upper(89).
is_upper(90).
